@@ -3,12 +3,12 @@
 #
 # Fill in `templates'
 #
-# Copyright 1996, 1997, 1999 M-J. Dominus.
+# Copyright 1996, 1997, 1999, 2001, 2002, 2003, 2008 M-J. Dominus.
 # You may copy and distribute this program under the
 # same terms as Perl iteself.  
-# If in doubt, write to mjd-perl-template+@pobox.com for a license.
+# If in doubt, write to mjd-perl-template+@plover.com for a license.
 #
-# Version 1.31
+# Version 1.45
 
 package Text::Template;
 require 5.004;
@@ -18,7 +18,7 @@ use Exporter;
 use vars '$ERROR';
 use strict;
 
-$Text::Template::VERSION = '1.31';
+$Text::Template::VERSION = '1.45';
 my %GLOBAL_PREPEND = ('Text::Template' => '');
 
 sub Version {
@@ -52,8 +52,10 @@ sub always_prepend
     my %a = @_;
     my $stype = uc(_param('type', %a)) || 'FILE';
     my $source = _param('source', %a);
+    my $untaint = _param('untaint', %a);
     my $prepend = _param('prepend', %a);
     my $alt_delim = _param('delimiters', %a);
+    my $broken = _param('broken', %a);
     unless (defined $source) {
       require Carp;
       Carp::croak("Usage: $ {pack}::new(TYPE => ..., SOURCE => ...)");
@@ -63,13 +65,19 @@ sub always_prepend
       Carp::croak("Illegal value `$stype' for TYPE parameter");
     }
     my $self = {TYPE => $stype,
-		SOURCE => $source,
 		PREPEND => $prepend,
+                UNTAINT => $untaint,
+                BROKEN => $broken,
 		(defined $alt_delim ? (DELIM => $alt_delim) : ()),
 	       };
+    # Under 5.005_03, if any of $stype, $prepend, $untaint, or $broken
+    # are tainted, all the others become tainted too as a result of
+    # sharing the expression with them.  We install $source separately
+    # to prevent it from acquiring a spurious taint.
+    $self->{SOURCE} = $source;
 
     bless $self => $pack;
-    #  $self->compile;
+    return unless $self->_acquire_data;
     
     $self;
   }
@@ -81,23 +89,30 @@ sub _acquire_data {
   my ($self) = @_;
   my $type = $self->{TYPE};
   if ($type eq 'STRING') {
-    return 1;
+    # nothing necessary    
   } elsif ($type eq 'FILE') {
-    my $text = _load_text($self->{SOURCE});
-    unless (defined $text) {
+    my $data = _load_text($self->{SOURCE});
+    unless (defined $data) {
       # _load_text already set $ERROR
       return undef;
     }
+    if ($self->{UNTAINT} && _is_clean($self->{SOURCE})) {
+      _unconditionally_untaint($data);
+    }
     $self->{TYPE} = 'STRING';
-    $self->{SOURCE} = $text;
+    $self->{FILENAME} = $self->{SOURCE};
+    $self->{SOURCE} = $data;
   } elsif ($type eq 'ARRAY') {
     $self->{TYPE} = 'STRING';
     $self->{SOURCE} = join '', @{$self->{SOURCE}};
   } elsif ($type eq 'FILEHANDLE') {
     $self->{TYPE} = 'STRING';
     local $/;
-    local *FH = $self->{SOURCE};
-    my $data = <FH>; # Extra assignment avoids bug in Solaris perl5.00[45].
+    my $fh = $self->{SOURCE};
+    my $data = <$fh>; # Extra assignment avoids bug in Solaris perl5.00[45].
+    if ($self->{UNTAINT}) {
+      _unconditionally_untaint($data);
+    }
     $self->{SOURCE} = $data;
   } else {
     # This should have been caught long ago, so it represents a 
@@ -114,6 +129,14 @@ sub source {
   return $self->{SOURCE};
 }
 
+sub set_source_data {
+  my ($self, $newdata) = @_;
+  $self->{SOURCE} = $newdata;
+  $self->{DATA_ACQUIRED} = 1;
+  $self->{TYPE} = 'STRING';
+  1;
+}
+
 sub compile {
   my $self = shift;
 
@@ -224,11 +247,14 @@ sub fill_in {
 
   my $fi_varhash = _param('hash', %fi_a);
   my $fi_package = _param('package', %fi_a) ;
-  my $fi_broken  = _param('broken', %fi_a)  || \&_default_broken;
+  my $fi_broken  = 
+    _param('broken', %fi_a)  || $fi_self->{BROKEN} || \&_default_broken;
   my $fi_broken_arg = _param('broken_arg', %fi_a) || [];
   my $fi_safe = _param('safe', %fi_a);
   my $fi_ofh = _param('output', %fi_a);
   my $fi_eval_package;
+  my $fi_scrub_package = 0;
+  my $fi_filename = _param('filename') || $fi_self->{FILENAME} || 'template';
 
   my $fi_prepend = _param('prepend', %fi_a);
   unless (defined $fi_prepend) {
@@ -241,6 +267,7 @@ sub fill_in {
     $fi_eval_package = $fi_package;
   } elsif (defined $fi_varhash) {
     $fi_eval_package = _gensym();
+    $fi_scrub_package = 1;
   } else {
     $fi_eval_package = caller;
   }
@@ -276,10 +303,13 @@ sub fill_in {
       }
     } elsif ($fi_type eq 'PROG') {
       no strict;
-      my $fi_progtext = "package $fi_eval_package; $fi_prepend;\n#line 1\n$fi_text";
+      my $fi_lcomment = "#line $fi_lineno $fi_filename";
+      my $fi_progtext = 
+        "package $fi_eval_package; $fi_prepend;\n$fi_lcomment\n$fi_text;";
       my $fi_res;
       my $fi_eval_err = '';
       if ($fi_safe) {
+        $fi_safe->reval(q{undef $OUT});
 	$fi_res = $fi_safe->reval($fi_progtext);
 	$fi_eval_err = $@;
 	my $OUT = $fi_safe->reval('$OUT');
@@ -322,6 +352,8 @@ sub fill_in {
       die "Can't happen error #2";
     }
   }
+
+  _scrubpkg($fi_eval_package) if $fi_scrub_package;
   defined $fi_ofh ? 1 : $fi_r;
 }
 
@@ -356,8 +388,9 @@ sub _default_broken {
   my $prog_text = $a{text};
   my $err = $a{error};
   my $lineno = $a{lineno};
-  $err =~ s/\s+at .*//s;
-  "Program fragment at line $lineno delivered error ``$err''";
+  chomp $err;
+#  $err =~ s/\s+at .*//s;
+  "Program fragment delivered error ``$err''";
 }
 
 sub _load_text {
@@ -371,11 +404,31 @@ sub _load_text {
   <F>;
 }
 
+sub _is_clean {
+  my $z;
+  eval { ($z = join('', @_)), eval '#' . substr($z,0,0); 1 }   # LOD
+}
+
+sub _unconditionally_untaint {
+  for (@_) {
+    ($_) = /(.*)/s;
+  }
+}
+
 {
   my $seqno = 0;
   sub _gensym {
     __PACKAGE__ . '::GEN' . $seqno++;
   }
+  sub _scrubpkg {
+    my $s = shift;
+    $s =~ s/^Text::Template:://;
+    no strict 'refs';
+    my $hash = $Text::Template::{$s."::"};
+    foreach my $key (keys %$hash) {
+      undef $hash->{$key};
+    }
+  }
 }
   
 # Given a hashful of variables (or a list of such hashes)
@@ -384,7 +437,7 @@ sub _load_text {
 sub _install_hash {
   my $hashlist = shift;
   my $dest = shift;
-  if (ref $hashlist eq 'HASH') {
+  if (UNIVERSAL::isa($hashlist, 'HASH')) {
     $hashlist = [$hashlist];
   }
   my $hash;
@@ -416,21 +469,21 @@ Text::Template - Expand template text with embedded Perl
 
 =head1 VERSION
 
-This file documents C<Text::Template> version B<1.31>
+This file documents C<Text::Template> version B<1.45>
 
 =head1 SYNOPSIS
 
  use Text::Template;
 
 
- $template = new Text::Template (TYPE => FILE,  SOURCE => 'filename.tmpl');
- $template = new Text::Template (TYPE => ARRAY, SOURCE => [ ... ] );
- $template = new Text::Template (TYPE => FILEHANDLE, SOURCE => $fh );
- $template = new Text::Template (TYPE => STRING, SOURCE => '...' );
- $template = new Text::Template (PREPEND => q{use strict;}, ...);
+ $template = Text::Template->new(TYPE => 'FILE',  SOURCE => 'filename.tmpl');
+ $template = Text::Template->new(TYPE => 'ARRAY', SOURCE => [ ... ] );
+ $template = Text::Template->new(TYPE => 'FILEHANDLE', SOURCE => $fh );
+ $template = Text::Template->new(TYPE => 'STRING', SOURCE => '...' );
+ $template = Text::Template->new(PREPEND => q{use strict;}, ...);
 
  # Use a different template file syntax:
- $template = new Text::Template (DELIMITERS => [$open, $close], ...);
+ $template = Text::Template->new(DELIMITERS => [$open, $close], ...);
 
  $recipient = 'King';
  $text = $template->fill_in();  # Replaces `{$recipient}' with `King'
@@ -439,11 +492,20 @@ This file documents C<Text::Template> version B<1.31>
  $T::recipient = 'Josh';
  $text = $template->fill_in(PACKAGE => T);
 
- $hash = { recipient => 'Abed-Nego' };
- $text = $template->fill_in(HASH => $hash, ...); # Recipient is Abed-Nego
+ # Pass many variables explicitly
+ $hash = { recipient => 'Abed-Nego',
+           friends => [ 'me', 'you' ],
+           enemies => { loathsome => 'Bill Gates',
+                        fearsome => 'Larry Ellison' },
+         };
+ $text = $template->fill_in(HASH => $hash, ...);
+ # $recipient is Abed-Nego,
+ # @friends is ( 'me', 'you' ),
+ # %enemies is ( loathsome => ..., fearsome => ... )
+
 
  # Call &callback in case of programming errors in template
- $text = $template->fill_in(BROKEN => \&callback, BROKEN_ARG => [...]);
+ $text = $template->fill_in(BROKEN => \&callback, BROKEN_ARG => $ref, ...);
 
  # Evaluate program fragments in Safe compartment with restricted permissions
  $text = $template->fill_in(SAFE => $compartment, ...);
@@ -523,7 +585,7 @@ template into the example result, and prints it out:
 
 	use Text::Template;
 
-	my $template = new Text::Template (SOURCE => 'formletter.tmpl')
+	my $template = Text::Template->new(SOURCE => 'formletter.tmpl')
 	  or die "Couldn't construct template: $Text::Template::ERROR";
 
 	my @monthname = qw(January February March April May June
@@ -626,7 +688,7 @@ into the last line.  The output will look something like this:
 	The Lord High Chamberlain has gotten 42
 	things for me this year.  
 
-	That is 35 more than he gave me last year.
+	That is 25 more than he gave me last year.
 
 That is all the syntax there is.  
 
@@ -751,6 +813,39 @@ string is the string that signals the beginning of each program
 fragment, and the second string is the string that signals the end of
 each program fragment.  See L<"Alternative Delimiters">, below.
 
+=item C<UNTAINT>
+
+If your program is running in taint mode, you may have problems if
+your templates are stored in files.  Data read from files is
+considered 'untrustworthy', and taint mode will not allow you to
+evaluate the Perl code in the file.  (It is afraid that a malicious
+person might have tampered with the file.)
+
+In some environments, however, local files are trustworthy.  You can
+tell C<Text::Template> that a certain file is trustworthy by supplying
+C<UNTAINT =E<gt> 1> in the call to C<new>.  This will tell
+C<Text::Template> to disable taint checks on template code that has
+come from a file, as long as the filename itself is considered
+trustworthy.  It will also disable taint checks on template code that
+comes from a filehandle.  When used with C<TYPE =E<gt> 'string'> or C<TYPE
+=E<gt> 'array'>, it has no effect.
+
+See L<perlsec> for more complete information about tainting.
+
+Thanks to Steve Palincsar, Gerard Vreeswijk, and Dr. Christoph Baehr
+for help with this feature.
+
+=item C<PREPEND>
+
+This option is passed along to the C<fill_in> call unless it is
+overridden in the arguments to C<fill_in>.  See L<C<PREPEND> feature
+and using C<strict> in templates> below.
+
+=item C<BROKEN>
+
+This option is passed along to the C<fill_in> call unless it is
+overridden in the arguments to C<fill_in>.  See L<C<BROKEN>> below.
+
 =back
 
 =head2 C<compile>
@@ -790,7 +885,7 @@ may yield spurious warnings about
 
 so you might like to avoid them and use the capitalized versions.
 
-At present, there are seven legal options:  C<PACKAGE>, C<BROKEN>,
+At present, there are eight legal options:  C<PACKAGE>, C<BROKEN>,
 C<BROKEN_ARG>, C<SAFE>, C<HASH>, C<OUTPUT>, and C<DELIMITERS>.
 
 =over 4
@@ -873,11 +968,20 @@ The value for C<HASH> should be a reference to a hash that maps
 variable names to values.  For example, 
 
 	$template->fill_in(HASH => { recipient => "The King",
-				     items => ['gold', 'frankincense', 'myrrh']
+				     items => ['gold', 'frankincense', 'myrrh'],
+	                             object => \$self,
 				   });
 
 will fill out the template and use C<"The King"> as the value of
-C<$recipient> and the list of items as the value of C<@items>.
+C<$recipient> and the list of items as the value of C<@items>.  Note
+that we pass an array reference, but inside the template it appears as
+an array.  In general, anything other than a simple string or number
+should be passed by reference.
+
+We also want to pass an object, which is in C<$self>; note that we
+pass a reference to the object, C<\$self> instead.  Since we've passed
+a reference to a scalar, inside the template the object appears as
+C<$object>.  
 
 The full details of how it works are a little involved, so you might
 want to skip to the next section.
@@ -898,6 +1002,8 @@ value in the template.
 
 =item *
 
+For anything else, you must pass a reference.
+
 If the I<value> is a reference to an array, then C<@key> is set to
 that array.  If the I<value> is a reference to a hash, then C<%key> is
 set to that hash.  Similarly if I<value> is any other kind of
@@ -913,6 +1019,18 @@ have almost exactly the same effect.  (The difference is that in the
 former case, the value is copied, and in the latter case it is
 aliased.)  
 
+=item *
+
+In particular, if you want the template to get an object or any kind,
+you must pass a reference to it:
+
+	$template->fill_in(HASH => { database_handle => \$dbh, ... });
+
+If you do this, the template will have a variable C<$database_handle>
+which is the database handle object.  If you leave out the C<\>, the
+template will have a hash C<%database_handle>, which exposes the
+internal structure of the database handle object; you don't want that.
+
 =back
 
 Normally, the way this works is by allocating a private package,
@@ -967,18 +1085,19 @@ fragment that cased the error.
 If you don't specify a C<BROKEN> function, C<Text::Template> supplies
 a default one that returns something like
 
-	Program fragment at line 17 delivered error ``Illegal
-	division by 0''
+	Program fragment delivered error ``Illegal division by 0 at
+	template line 37''
 
-Since this is interpolated into the template at the place the error
-occurred, a template like this one:
+(Note that the format of this message has changed slightly since
+version 1.31.)  The return value of the C<BROKEN> function is
+interpolated into the template at the place the error occurred, so
+that this template:
 
 	(3+4)*5 = { 3+4)*5 }
 
 yields this result:
 
-	(3+4)*5 = Program fragment at line 1 delivered error
-	``syntax error''
+	(3+4)*5 = Program fragment delivered error ``syntax error at template line 1''
 
 If you specify a value for the C<BROKEN> attribute, it should be a
 reference to a function that C<fill_in> can call instead of the
@@ -995,12 +1114,16 @@ The source code of the program fragment that failed
 
 =item C<error>
 
-The text of the error message (C<$@>) generated by eval
+The text of the error message (C<$@>) generated by eval.
+
+The text has been modified to omit the trailing newline and to include
+the name of the template file (if there was one).  The line number
+counts from the beginning of the template, not from the beginning of
+the failed program fragment.
 
 =item C<lineno>
 
-The line number of the template data at which the  program fragment
-began
+The line number of the template at which the program fragment began.
 
 =back
 
@@ -1327,10 +1450,9 @@ It may be safer to choose delimiters that begin with a newline
 character.  
 
 Because the parsing of templates is simplified by the absence of
-backslash escapes, using alternative C<DELIMITERS> I<speeds up> the
+backslash escapes, using alternative C<DELIMITERS> may speed up the
 parsing process by 20-25%.  This shows that my original choice of C<{>
-and C<}> was very bad.  I therefore recommend that you use alternative
-delimiters whenever possible. 
+and C<}> was very bad. 
 
 =head2 C<PREPEND> feature and using C<strict> in templates
 
@@ -1549,26 +1671,22 @@ interpolated.  This is what is known as a `trick'.
 =head2 Compatibility
 
 Every effort has been made to make this module compatible with older
-versions.  There are three exceptions.  One is the output format of
-the default C<BROKEN> subroutine; I decided that the old format was
-too verbose.  If this bothers you, it's easy to supply a custom
-subroutine that yields the old behavior.  The second is that the
-C<$OUT> feature arrogates the C<$OUT> variable for itself.  If you had
-templates that happened to use a variable named C<$OUT>, you will have
-to change them to use some other variable or all sorts of strangeness
-may result.
+versions.  The only known exceptions follow:
 
-The third incompatibility is with the behavior of the \ metacharacter.
-In 0.1b, \\ was special everywhere, and the template processor always
-replaced it with a single backslash before passing the code to Perl
-for evaluation.  The rule now is more complicated but probably more
-convenient.  See the section on backslash processing, below, for a
-full discussion.
+The output format of the default C<BROKEN> subroutine has changed
+twice, most recently between versions 1.31 and 1.40.
 
-With a minor change to fix the format of the default C<BROKEN>
-subroutine, this version passes the test suite from the old version.
-(It is in C<t/01-basic.t>.) The old test suite was too small, but it's
-a little reassuring.
+Starting in version 1.10, the C<$OUT> variable is arrogated for a
+special meaning.  If you had templates before version 1.10 that
+happened to use a variable named C<$OUT>, you will have to change them
+to use some other variable or all sorts of strangeness will result.
+
+Between versions 0.1b and 1.00 the behavior of the \ metacharacter
+changed.  In 0.1b, \\ was special everywhere, and the template
+processor always replaced it with a single backslash before passing
+the code to Perl for evaluation.  The rule now is more complicated but
+probably more convenient.  See the section on backslash processing,
+below, for a full discussion.
 
 =head2 Backslash Processing
 
@@ -1579,9 +1697,9 @@ generate a backslash, because the code passed to Perl for evaluation
 was C<"\"> which is a syntax error.  If you wanted a backslash, you
 would have had to write C<{"\\\\"}>.
 
-In C<Text::Template> versions 1.0 through 1.10, there was a bug:
+In C<Text::Template> versions 1.00 through 1.10, there was a bug:
 Backslash was special everywhere.  In these versions, C<{"\n"}>
-generated the letter C<n>.  
+generated the letter C<n>.
 
 The bug has been corrected in version 1.11, but I did not go back to
 exactly the old rule, because I did not like the idea of having to
@@ -1670,13 +1788,17 @@ inside the template:
 			    );
 	}
 
+=head2 Automatic preprocessing of program fragments
+
+It may be useful to preprocess the program fragments before they are
+evaluated.  See C<Text::Template::Preprocess> for more details.
 
 =head2 Author
 
 Mark-Jason Dominus, Plover Systems
 
 Please send questions and other remarks about this software to
-C<mjd-perl-template@pobox.com>
+C<mjd-perl-template+@plover.com>
 
 You can join a very low-volume (E<lt>10 messages per year) mailing
 list for announcements about this package.  Send an empty note to
@@ -1686,60 +1808,107 @@ For updates, visit C<http://www.plover.com/~mjd/perl/Template/>.
 
 =head2 Support?
 
-This software is version 1.0.  It is a complete rewrite of an older
-package, and may have bugs.  Suggestions and bug reports are always
-welcome.  Send them to C<mjd-perl-template@plover.com>.  (That is my
-address, not the address of the mailing list.  The mailing list
-address is a secret.)
+This software is version 1.45.  It may have bugs.  Suggestions and bug
+reports are always welcome.  Send them to
+C<mjd-perl-template+@plover.com>.  (That is my address, not the address
+of the mailing list.  The mailing list address is a secret.)
 
-=head2 Thanks
+=head1 LICENSE
+
+    Text::Template version 1.45
+    Copyright (C) 2008 Mark Jason Dominus
+
+    This program is free software; you can redistribute it and/or
+    modify it under the terms of the GNU General Public License as
+    published by the Free Software Foundation; either version 2 of the
+    License, or (at your option) any later version.  You may also can
+    redistribute it and/or modify it under the terms of the Perl
+    Artistic License.
+
+    This program is distributed in the hope that it will be useful,
+    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+    GNU General Public License for more details.
+
+    You should have received copies of the GNU General Public License
+    along with this program; if not, write to the Free Software
+    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+
+
+=head1 THANKS
 
 Many thanks to the following people for offering support,
-encouragement, advice, and all the other good stuff.  
+encouragement, advice, bug reports, and all the other good stuff.  
 
+David H. Adler /
+Joel Appelbaum /
 Klaus Arnhold /
+AntE<oacute>nio AragE<atilde>o /
 Kevin Atteson /
 Chris.Brezil /
 Mike Brodhead /
 Tom Brown /
+Dr. Frank Bucolo /
 Tim Bunce /
 Juan E. Camacho /
+Itamar Almeida de Carvalho /
 Joseph Cheek /
+Gene Damon /
 San Deng /
 Bob Dougherty /
+Marek Grac /
 Dan Franklin /
+gary at dls.net /
 Todd A. Green /
 Donald L. Greer Jr. /
 Michelangelo Grigni /
+Zac Hansen /
 Tom Henry /
+Jarko Hietaniemi /
 Matt X. Hunter /
 Robert M. Ioffe /
 Daniel LaLiberte /
 Reuven M. Lerner /
+Trip Lilley / 
+Yannis Livassof /
+Val Luck /
+Kevin Madsen /
+David Marshall /
+James Mastros /
 Joel Meulenberg /
 Jason Moore /
+Sergey Myasnikov /
+Chris Nandor /
 Bek Oberin /
+Steve Palincsar /
 Ron Pero /
 Hans Persson /
+Sean Roehnelt /
 Jonathan Roy /
 Shabbir J. Safdar /
 Jennifer D. St Clair /
 Uwe Schneider /
 Randal L. Schwartz /
 Michael G Schwern /
+Yonat Sharon /
 Brian C. Shensky /
 Niklas Skoglund /
 Tom Snee /
+Fred Steinberg /
 Hans Stoop /
 Michael J. Suzio /
 Dennis Taylor /
 James H. Thompson /
 Shad Todd /
+Lieven Tomme /
 Lorenzo Valdettaro /
 Larry Virden /
 Andy Wardley /
+Archie Warnock /
+Chris Wesley /
 Matt Womer /
 Andrew G Wood /
+Daini Xie /
 Michaely Yeung
 
 Special thanks to:
@@ -1770,9 +1939,6 @@ The line number information will be wrong if the template's lines are
 not terminated by C<"\n">.  You should let me know if this is a
 problem.  If you do, I will fix it.
 
-The default format for reporting of broken program fragments has
-changed since version 0.1.  
-
 The C<$OUT> variable has a special meaning in templates, so you cannot
 use it as if it were a regular variable.
 
